// ATtiny85 timing using timer1 interrupt

// Started: Feb 27, 2015  G. D. (Joe) Young <jyoung@islandnet.com>
//
// Revised: Mar 01/15 - code output during on time - codppm
//          Mar 06/15 - correct logic, add letter spacing
//                    - try several OSCCAL, OCR1C values - see notes
//          Mar 08/15 - codppmx - 1 MHz crystal processor osc
//                    - needs lfuse changed from 0x62 to 0xeb
//                    - OCR1C theoretically correct value 243 is best
//                    - add trimming adjustment to int service

#include <avr/io.h>
#include <avr/interrupt.h>

#define THISFOX 3
#define TRIMCNT 5

// interrupt service variables
volatile word ovflcnt = 0;
volatile byte trimcnt;
word loopovfl;

byte ledPin = 0;
byte timerPin = 1;    // timer compare toggles OC1A on PB1
boolean code = false; // true during code sending
boolean sendm, sendid, sendsp;
boolean sendlsp, sendwsp, eltOn;  // loop control of morse output
unsigned long timer;
unsigned long dotLen = 120;  // dot length in millisec (1200/wpm)
byte dash, id;


// timer overflow interrupt
ISR( TIMER1_OVF_vect ) {
  ovflcnt += 1;
  trimcnt -= 1;
  if( trimcnt == 1 ) OCR1C = 244;
  if( trimcnt == 0 ) {
    trimcnt = TRIMCNT;
    OCR1C = 243;
  }
} // timer count overflow interrupt service

void setupTimer1( ) {
  TCCR1 = bit(CTC1) | bit(PWM1A) | bit(COM1A1) | 0x0d;
//  OCR1C = 250;    // max tcnt to reset internal osc
//  OCR1C = 251;    // max tcnt to reset
  OCR1C = 243;    // 243 best for 1 MHz xtal osc
  OCR1A = 120;    // adj duty cycle pb1 toggles at this tcnt
  TIMSK |= bit(TOIE1);  //add timer1 overflow interrupt
} // setupTimer1 for interrupt generation on overflow

void setup( ) {
  noInterrupts( );    // attempt to sync millis( ) with timer1
//  OSCCAL -= 1;
  setupTimer1( );
  trimcnt = TRIMCNT;        // allow fine adjustment of timing in int svc
  interrupts( );
  pinMode( ledPin, OUTPUT );
  pinMode( timerPin, OUTPUT );
  loopovfl = ovflcnt+1;
  digitalWrite( ledPin, LOW );
  eltOn = false;
} // setup( )


void loop( ) {
  if( !code && loopovfl < ovflcnt ) {  // start sending
    loopovfl += 12;
//    loopovfl += 40;    // for testing, on for longer.
    sendm = true;
    dash = 2;
    code = true;
  }
  if( code && loopovfl < ovflcnt ) {    // end sending
    loopovfl += 48;
//    loopovfl += 20;    // for testing
    digitalWrite( ledPin, LOW );
    code = false;
  }
  
  if( code && sendm && !eltOn && ( dash > 0 ) ) {
    timer = millis( ) + 3*dotLen;  // sending a dash
    digitalWrite( ledPin, HIGH );
    eltOn = true;
    dash--;
    if( dash == 0 ) {
      sendm = false;
      sendlsp = true;
    } // after m, send letter space
  } // if sending m

  if( code && eltOn && !sendsp && ( timer < millis( ) ) ) {
    digitalWrite( ledPin, LOW );
    timer += dotLen;    // timing space following element
    sendsp = true;
    if( sendlsp ) timer += 2*dotLen;
    if( sendwsp ) timer += 4*dotLen;
  } // if element on time is up
  if( code && sendsp && ( timer < millis( ) ) ) {
    eltOn = false;
    sendsp = false;
    if( sendlsp ) {
      sendid = true;   //cue sending Id
      id = THISFOX;        // this will be read from pins
      sendlsp = false;
    } // if space was letter space
    if( sendwsp ) {
      sendm = true;
      dash = 2;
      sendwsp = false;
    }
  } // if sending space 

  if( code && sendid && !eltOn && id > 0 ) {
    timer =  millis( ) + dotLen;
    digitalWrite( ledPin, HIGH );
    eltOn = true;
    id--;
    if( id == 0 ) {
      sendwsp = true;
      sendid = false;
    }
  } // if sending dots for id
  
  
} // loop( )


