// controller memory tests - condev --> condev2 for controller with callsign ID
//
// started: June 9, 2015  G. D. (Joe) Young <jyoung@islandnet.com>
//
// revised: Jun 11/15 - save freeRam value at setup in EEPROM 0,1
//          Sep 08/15 - modify for sending code patterns stored in EEPROM
//          Sep 10/15 - add callsign insertion, remove freeRam testing
//
// Start from codppmx7, the ATtiny85 version moved to run in regular
// arduino UNO:
//  outputs on 13, 12
//  switch timing to just use millis( ) for development testing
//


//#include <avr/io.h>
//#include <avr/interrupt.h>

#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <EEPROM.h>
#include <codeTable.h>    // just to get EE written first time
#include <ttMorse.h>


#define CALL_INTERVAL 120  // time interval between sending callsign
//#define THISFOX 1   // now lookup fox number from eeprom

#define I2CADDR 0x38  // keypad on PCF8574A
#define LCD_ADR 0x20  // set the LCD address to 0x20
#define LCD_CHRS 16   // for a 16 chars ...
#define LCD_LINS 2    //   and 2 line display

// use lcd for testing messages
LiquidCrystal_I2C lcd( LCD_ADR, LCD_CHRS, LCD_LINS );



// interrupt service variables
volatile word ovflcnt = 0;
word loopovfl;

const byte ledPin = 13;
const byte txPin = 12;
const byte ainPin = A1;

// eeprom contents - will translate to code bytes, eg. 'M' --> 0b00000111
byte tid[ ] = { 'M','O','S',' ','D','E',' ','V','E','7','B','F','K','/','7',0xff };

byte fmode, thisfox;
word onTime, offTime, ledtime, calltime;
boolean code = false;             // true during code sending
boolean sendm, sendo, msent, sendid, sendsp, cend, ledon;
boolean sendlsp, sendwsp, eltOn;  // loop control of morse output
//unsigned long timer;
unsigned long timer, sectimer;
//unsigned long dotLen = 221;  // dot length in millisec (1200/wpm)1.8432
unsigned long dotLen = 150;  // dot length in millisec (1200/wpm)1.8432
unsigned long cdotLen = 40;  // dot length while sending call sign
unsigned long savedotLen;
byte dash, id;

// change interrupt service to compare match interrupt
//ISR( TIM1_COMPA_vect ) {
//  ovflcnt += 1;
//}

//void setupTimer1( ) {
//  TCCR1 = bit(CTC1) | 0x0e; // do not use pwm mode
//  OCR1C = 224;    // 225 div with 1.8432 MHz xtal
//  OCR1A = 224;    // using compare match int
//  TIMSK |= bit(OCIE1A);  //add timer1 overflow interrupt
//} // setupTimer1 for interrupt generation on overflow


void setupNext( byte );       // functions to setup pointers to code bytes
void setupCall( );
byte setGetBit( );    // code byte parsing, now returns fence bit position 7..0
bool getBit( );
byte codeChar, nextelt, nextBit, bitCnt, nextChar, thisLength;
byte callend;
bool endCode, callsend;

void setup( ) {

  int temp = analogRead( ainPin );      // get operating mode
  fmode = temp>>8;
  
  onTime = 12;
  offTime = 48;
//  if( fmode == 1 ) dotLen -= 64;        // fast sprint
  if( fmode == 1 ) dotLen -= 30;        // fast sprint
  savedotLen = dotLen;
  if( fmode == 3 ) {
    onTime = 60;
    offTime = 240;
  } // if normal timing
//  noInterrupts( );    // attempt to sync millis( ) with timer1
//  setupTimer1( );
//  interrupts( );
  sectimer = millis( ); 
  pinMode( ledPin, INPUT );  // setup for open drain o/p
  digitalWrite( ledPin, LOW ); // will setup no pullups, 0 in reg.
  digitalWrite( txPin, LOW );
  eltOn = false;
  cend = false;
  ledon = true;

  codeChar = EEPROM.read( 2 );  //determine fox number from EE code character
  thisfox = setGetBit( );
  callend = 4;
  while( EEPROM.read( callend ) != 0xff ) callend++;
//  callend -= 2;               //not needed if 0xff follows last code byte

  thisLength = 3;
  if( fmode == 2 ) {
    thisfox = 0;              // no fox id for continuous
    thisLength = 2;
    ledtime = ovflcnt + 30;   // 30 sec for led 
  }
  lcd.init( );
  lcd.backlight( );
  lcd.print( "mode " );
  lcd.print( fmode );
  lcd.print( " fox " );
  lcd.print( thisfox );
//  lcd.clear( );
//  EEPROM.write( 0, 0xff );
  if( EEPROM.read( 0 ) != morseTable[ 'M' - 0x20 ] ) {
    lcd.print( "writing EEPROM" );
    delay( 1000 );
    for( byte ix=0; ix < sizeof( tid ); ix++ ) {
      EEPROM.write( ix, morseTable[ tid[ix] - 0x20 ] );
    } // loop over id text, convert to code bytes
  } // if EEPROM not setup
//  lcd.clear( );

// setup timers
  if( fmode == 0 || fmode == 1 ) {
    loopovfl = ovflcnt+1+(thisfox-1)*onTime;  //indexed start for sprint
  } else {
    loopovfl = ovflcnt + 1;   // no indexing for normal
  } // if sprint modes

  calltime = ovflcnt+2;

} // setup( )


void loop( ) {
  
  //seconds timing using millis( )
  if( millis( ) - sectimer > 1000 ) {
    ovflcnt += 1;
    sectimer = millis( );
  }

  
  if( !code && loopovfl == ovflcnt ) {  // start sending
    loopovfl += onTime;
    if( fmode == 3 ) {
      setupCall( );               // mode 3 send call every start
    } else {
      dotLen = savedotLen;
      nextChar = 0;
      setupNext( nextChar );      // get first letter of fox code
    }
    code = true;
    cend = false;
  }

  if( code && (loopovfl == ovflcnt) && (fmode != 2) ) {    // end sending
    loopovfl += offTime;
    cend = true;    // delay end until id complete
  }
  
  if( fmode==2 && ledtime == ovflcnt ) ledon = false;

//  if( fmode == 2 && (ovflcnt-calltime) > CALL_INTERVAL ) {
  if( fmode != 3 && (ovflcnt-calltime) > CALL_INTERVAL ) {
    calltime = ovflcnt;
    callsend = true;
  }

  if( code && !eltOn ) {
    timer = millis( ) + dotLen;            // setup for dot length
    if( getBit( ) ) timer += 2*dotLen;      // sending a dash
    pinMode( txPin, OUTPUT );
    if( ledon ) pinMode( ledPin, OUTPUT );
    eltOn = true;
  } // if not sending, start first element

  if( code && eltOn && !sendsp && ( timer < millis( ) ) ) {
    pinMode( txPin, INPUT );
    if( ledon ) pinMode( ledPin, INPUT );
    timer += dotLen;    // timing space following element
    sendsp = true;
    if( endCode ) {
      endCode = false;
      sendlsp = true;
      timer += 2*dotLen;
      nextChar++;
      if( (nextChar == thisLength) || (nextChar == callend) ) {
        dotLen = savedotLen;
        sendwsp = true;
        timer += 3*dotLen;
        nextChar = 0;          // reset pointer to characters
      } // if last character of this group was just sent
      setupNext( nextChar );
      if( codeChar == 0 ) {
        sendwsp = true;
        timer += 3*dotLen;
        nextChar++;
        setupNext( nextChar );    // if space charcter to be sent
      }
    } // extend space to letter space or word space
  } // if element on time is up

  if( code && sendsp && ( timer < millis( ) ) ) {
    eltOn = false;
    sendsp = false;
    if( sendlsp ) {
      sendlsp = false;
    } // if letter space just ended
    if( sendwsp ) {
      sendwsp = false;
      if( cend ) {
        code = false;  // turn off sending
        ledon = false;   // and led after first burst
      } // if last word space, end sending
//      if( fmode == 2 && callsend ) {
      if( fmode != 3 && callsend ) {
        callsend = false;
        setupCall( );
      } // if time to send a callsign
    } // if word space ended, setup to begin again
  } // if sending space 

  
} // loop( )

void setupNext( byte nextChar ) {
  codeChar = EEPROM.read( nextChar );
  setGetBit( );
} // setup to first code byte (usually for M)

void setupCall( ) {
  nextChar =  4;    // start of callsign in EE
  codeChar = EEPROM.read( nextChar );
  setGetBit( );
  dotLen = cdotLen;
} // setup to first code byte of callsign ID

byte setGetBit( ) {
  bitCnt = 7;
  nextBit = 0b10000000;
  while( (bitCnt > 0) && ((nextBit & codeChar) == 0) ) {
    nextBit = nextBit>>1;
    bitCnt--;
  } // while leading zeros before fence bit
  return bitCnt;
} // setup the code word parsing

bool getBit( ) {
  nextBit = nextBit>>1;
  bitCnt--;
  if( bitCnt == 0 ) {
    endCode = true;
  } // if last bit has just been selected
  return( (nextBit & codeChar) == nextBit );
} // get next code element to send


