// text2code - prepare Morse code bytes from user input text
//
// Started: September 7, 2015  G. D. (Joe) Young <jyoung@islandnet.com>
//
// Revised: Sep 8/15 - use newline for end-of-input instead of \
//          Jan 24/16 - correct size of ihex[ ] allow input >15 characters
//
// A utility to prepare the data required for loading into the eeprom of
// the ATtiny85 minifox controller. Text input using serial monitor.
//
// Be sure the serial monitor is set to 'newline' for input terminator.

#include "codeTable.h"


char inputline[80];

byte lcount = 0;  // count of input characters
byte errfl = 0;   // return from parsing functions

char ihex[ 2*80+10 ];

byte cksum = 0;
byte codeChar;
bool high = true;
bool low = false;
boolean fatalerror = false;

char byte2hex( byte b, bool hi ) {
  if( hi ) {
    b = (b>>4)&0x0f;
  } else {
    b = b&0x0f;
  }
  if( b < 10 ) {
    return( 0x30 | b );
  } else {
    return( 0x41 + (b-10) );
  }
} // byte2hex( )


void setup( ) {
  Serial.begin( 9600 );
  inputline[0] = 0;
  Serial.println( "enter text string - example: MOE de ve7bfk/7" );
  Serial.println( "" );
} // setup

void loop() {
  if ( Serial.available( ) > 0 ) {
    while ( Serial.available( ) && lcount < (sizeof(inputline)) ) {
      inputline[lcount] = toupper( Serial.read( ) );
      if ( inputline[lcount] == '\n' ) break;
      Serial.print( inputline[lcount], HEX );
      Serial.print( ' ' );
      lcount++;
    } // while input
    if ( inputline[lcount] == '\n' ) {
      inputline[lcount] = 0;
      Serial.println( "   input as ASCII hex" );
      //      Serial.println( "" );
      for ( byte ix = 0; ix < lcount; ix++ ) {
        codeChar = morseTable[ inputline[ix] - 0x20 ];
        cksum += codeChar;
        ihex[2 * ix + 9] = byte2hex( codeChar, high ); //build ihex as translating
        ihex[2 * ix + 10] = byte2hex( codeChar, low );
//        Serial.print( codeChar, HEX );
        Serial.print( ihex[2*ix + 9] );
        Serial.print( ihex[2*ix + 10] );
        Serial.print( ' ' );
      }
      Serial.println( "   input as code bytes" );
      size_t ccount = strlen( inputline ); // omits terminator(s)
      Serial.print( inputline );
      Serial.print( "   input line as encoded   char count = " );
      Serial.println( ccount );
      Serial.println( "" );

      // Then finally emit intel-hex version
      // finish up the intel-hex string
      ihex[0] = ':';
      ihex[1] = byte2hex( ccount + 1, high ); //+1 to allow for fence byte
      ihex[2] = byte2hex( ccount + 1, low );
      cksum += ccount + 1;
      ihex[3] = '0';
      ihex[4] = '0';
      ihex[5] = '0';
      ihex[6] = '0';
      ihex[7] = '0';
      ihex[8] = '0';
      ihex[2 * ccount + 9] = 'F';    // terminator fence byte in case ee not unprogrammed
      ihex[2 * ccount + 10] = 'F';
      cksum += 0xff;
      ihex[11 + 2 * ccount] = byte2hex( ~cksum + 1, high );
      ihex[12 + 2 * ccount] = byte2hex( ~cksum + 1, low );
      ihex[13 + 2 * ccount] = 0;
      Serial.println( "intel-hex file format (with added 0xff terminator):" );
      Serial.println( ihex );

      // emit an end record to complete a valid intel-hex file
      Serial.println( ":00000001FF" );
      Serial.println( "" );
      Serial.println( "" );

      cksum = 0;
      lcount = 0;               // prepare to get a new line
    } // if end of line
  } // if input available


}
